/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};


// Wait for DOMContentLoaded event before trying to access the application template
var init = function () {

	let initComplete = false;
	templateObj = document.querySelector('#template_obj');

	// Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
	// before trying to access components in the application.
	templateObj.addEventListener('dom-change', function () {
		if (initComplete) return;
		this.async(function () {
			initComplete = true;

			// Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
			// e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId
			templateObj.$.ti_widget_toast.hideToast();
			templateObj.$.ti_widget_eventlog_view.log("info", "Application started");


			//
			// Developer mode configurations
			//
			const devMode = new Polymer.IronMetaQuery({ key: 'DEVELOPER_MODE' }).value;
			if (devMode) {
				//document.querySelector('ti-widget-readme')._dontShowAgain = true;
				//document.querySelector('ti-device-identifier').autoDetectDisabled = true;
				//document.querySelector('ti-device-identify-popup').suppressAllErrorPopups = true;
				//document.getElementById('ti_widget_vtabcontainer').selectedIndex = 1;

				// TODO: It would be nice to automatically load in a file an analyze it
				//const testData = "Desktop\\Projects\\data-analysis-library\\test_data\\ideal_24bit_-0.5dBFS_998.687744140625Hz_25kSPS_32768.csv";
			}


			//
			// Vertical tab container
			//
			const vtabcontainer = document.getElementById('ti_widget_vtabcontainer');
			if (vtabcontainer) {
				vtabcontainer.addEventListener("before_selected_tab_changed", (event) => {
					let selectedIndex = vtabcontainer.selectedIndex;
				// 	switch (selectedIndex) {			// TODO: Use labels instead of indices since they might change!
				// 		case 1:
				// 			// Redraw active plot
				// 			let dataDisplay = document.querySelector('ti-custom-data-display');
				// 			if (dataDisplay) { dataDisplay._onTabChanged(dataDisplay.analysisTab); }
				// 			break;
				// 	}
				});

				// Go to analysis tab
				templateObj.goToAnalysisTab = function () {
					vtabcontainer.selectedIndex = 1;
				};

				// Toggle expanded tab menu
				templateObj.toggleTabMenu = function () {
					templateObj.$.ti_widget_vtabcontainer.isExpanded = !templateObj.$.ti_widget_vtabcontainer.isExpanded;
				};

			}


			// open the eventlog viewer when the user clicks on the book icon in the status bar:
			templateObj.$.ti_widget_eventlog_view.closeView(); // ensure that view states are in sync with application
			templateObj.$.ti_widget_statusbar.addEventListener("appiconclicked", function (event) {
				templateObj.$.ti_widget_eventlog_view.toggleView();
			});

			//
			// Event log listeners
			//

			// TODO: Decide whether to enable these events in ti-sar-bindings or not
      		// // called when the user clicks the Write Register button:
			// document.addEventListener("write_reg", function (event) {
			// 	var detailsObj = event.detail;
			// 	templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
			// }.bind(this));

			// // called when the user clicks the Write All Registers button:
			// document.addEventListener("write_regs", function (event) {
			// 	var detailsObj = event.detail;
			// 	templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
			// }.bind(this));

			// // called when the user clicks the Read Register button:
			// document.addEventListener("read_reg", function (event) {
			// 	var detailsObj = event.detail;
			// 	templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
			// }.bind(this));

			// // called when the user clicks the Read All Registers button:
			// document.addEventListener("read_regs", function (event) {
			// 	var detailsObj = event.detail;
			// 	templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
			// }.bind(this));

			// // called when the user changes the immediate / deferred mode
			// document.addEventListener("update_mode_changed", function (event) {
			// 	var detailsObj = event.detail;
			// 	templateObj.$.ti_widget_eventlog_view.log("info", "request to change update mode to " + detailsObj.mode.label);
			// }.bind(this));

			// // called when the user changes the auto read refresh rate
			// document.addEventListener("auto_read_config_changed", function (event) {
			// 	var detailsObj = event.detail;
			// 	templateObj.$.ti_widget_eventlog_view.log("info", "request to change automatic read interval to " + detailsObj.autoRead.label);
			// }.bind(this));

			//
			// Template object functions
			//

			templateObj.showToastMsg = function (msgType, msg, durationInSeconds) {

				const toast = templateObj.$.ti_widget_toast;
				if (!toast) { return; }

				toast.duration = +durationInSeconds || 5;		// Default duratation if none is specified
				toast.hideToast();
				toast.message = msg;
				switch (msgType.toLowerCase()) {
					case "error":
						toast.iconName = "error";
						toast.backgroundColor = "red";
						break;
					case "warning":
						toast.iconName = "warning";
						toast.backgroundColor = "orange";
						break;
					default:
						toast.iconName = "check";
						toast.backgroundColor = "teal";
				}
				toast.showToast();
				console.log("toast msg [" + msgType + "]: " + msg);
			};

			templateObj.setStatus = function (msg, iconName) {

				const statusBar = document.querySelector('ti-widget-statusbar');
				if (!statusBar) { return; }

				iconName = iconName || templateObj.$.ti_widget_eventlog_view.iconForItem(msgType);
				statusBar.setIcon(iconName);

				statusBar.statusString3 = msg;
			};

			templateObj.logMsgAndNotify = function (msgType, msg, showInStatusBar, showInToast) {

				// Event log
				templateObj.$.ti_widget_eventlog_view.log(msgType, msg);

				// Toast message
				if (showInToast) { templateObj.showToastMsg(msgType, msg); }

				// Status message
				if (showInStatusBar) { templateObj.setStatus(msg); }

			};

			// Populate dropdown from register.json file
			// The register model reads in the .json file when it starts up, so in order to get
            // access to this information, we need to wait for the register model to send a registerDefsChanged
            // event that says it has updated its register model:
            registerModel = Polymer.dom(document).querySelector('ti-model-register');
            registerModel.addEventListener('registerDefsChanged', function(event) {
                // we then need to ask it for the JSON object that corresponds to the values in the register.json file:
                    
                    //registerModel.getRegisterViewInformation().then(function(deviceObj) {
                    // filter the register blocks based on id
                    // const regBlockArray = deviceObj.regblocks.filter((regblock) => (regblock.id === "configuration"));
                    // if (regBlockArray.length > 0) {
                    //     const regBlock = regBlockArray[0];
                    //     // filter the registers from that register block based on id
                    //     let regArray = regBlock.registers.filter((reg) => (reg.id === "config_register"));
                    //     if (regArray.length > 0) {
                    //         let reg = regArray[0];
                    //         // filter the bitfields from that register based on id
                    //         bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "adcrange"));
                    //         if (bitfieldArray.length > 0) {
                    //             let bitfield = bitfieldArray[0];
                    //             // update the description with GC specific notes
                    //             if(bitfield.desc.length < 200) //only add the note once...
                    //             {
                    //                 bitfield.desc += "\n\nNote: Changing this register value will update SHUNT_CAL, per the configuration in the Settings tab. This register can also be changed from the Settings tab.";
                    //             }
                                
                    //             // use the options for the bitfield to create the labels and values for the droplist
                    //             const widgetOptions = bitfield.widget.options;
                    //             const labels = widgetOptions.map((option) => (option.display));
                    //             templateObj.$.ti_widget_droplist_config_adcrange.labels = labels.join("|");
                    //             const values = widgetOptions.map((option) => (option.value));
                    //             templateObj.$.ti_widget_droplist_config_adcrange.values = values.join(",");
                    //             // the widget's binding to the register bitfield controls what the selected value and label is
                    //         }
                    //     }
                    //     // filter the registers from that register block based on id
                    //     regArray = regBlock.registers.filter((reg) => (reg.id === "shunt_cal_register"));
                    //     if (regArray.length > 0) {
                    //         reg = regArray[0];
                    //         // filter the bitfields from that register based on id
                    //         bitfieldArray = reg.fields.filter((bitfield) => (bitfield.id === "shunt_cal"));
                    //         if (bitfieldArray.length > 0) {
                    //             bitfield = bitfieldArray[0];
                    //             // update the description with GC specific notes
                    //             if(bitfield.desc.length < 350) //only add the note once...
                    //             {
                    //                 bitfield.desc += "\n\nNote: This register value will update when ADCRANGE is changed, per the configuration in the Settings tab.  This register can also be changed from the Settings tab.";
                    //             }
                    //         }
                    //     }
                    // }
                    
                });
            });

		}, 1);

	});

};

let templateObj = document.querySelector('#template_obj');
if (templateObj) {
	init();
} else {
	document.addEventListener('DOMContentLoaded', init);
}